from pyscript import document
from pyscript import window
import js
import sympy as sp
from sympy.parsing.sympy_parser import parse_expr

def longWeyl(size):
    return sp.Matrix([[1 if i + j == size - 1 else 0 for j in range(size)] for i in range(size)])

def lu_decomposition(g):
    dimension = g.shape[0]
    currentMatrix = g
    lower = sp.eye(dimension)
    upper = sp.eye(dimension)

    for j in range(dimension):
        pivotFound = False
        currentRow = 0
        while not pivotFound and currentRow < dimension:
            if currentMatrix[currentRow, j] == 0:
                currentRow += 1
            else:
                pivotFound = True
                pivotValue = currentMatrix[currentRow, j]
                currentLower = sp.eye(dimension)
                for i in range(currentRow + 1, dimension):
                    currentLower[i, currentRow] = currentMatrix[i, j] / pivotValue
                lower = lower @ currentLower

                currentUpper = sp.eye(dimension)
                for currentColumn in range(j + 1, dimension):
                    currentUpper[j, currentColumn] = currentMatrix[currentRow, currentColumn] / pivotValue

                upper = currentUpper @ upper
                currentMatrix = sp.simplify(sp.simplify(currentLower.inv() @ currentMatrix @ currentUpper.inv()))

    return currentMatrix, lower, upper

def bruhat_decomposition(g):
    dimension = g.shape[0]
    longWeylMat = longWeyl(dimension)
    dw, u1, u2 = lu_decomposition(longWeylMat @ g)
    u1 = longWeylMat @ u1 @ longWeylMat
    dw = longWeylMat @ dw
    return dw, u1, u2

def process_input(event):
    matrix_input = document.getElementById("inputData").value
    rows = matrix_input.split("\\\\")
    parsed = [list(map(lambda y: parse_expr(y.strip().replace("^","**"), evaluate=False), row.split("&"))) for row in rows]
    g = sp.Matrix(parsed)
    dw, u1, u2 = bruhat_decomposition(g)
    results = [sp.latex(dw), sp.latex(u1), sp.latex(u2)]
    table = document.getElementById("outputTable")
    table_html = f"<tr><td>$g = u_1 dw u_2$</td><td>${sp.latex(g)}$</td></tr>"
    table_html += f"<tr><td>$dw$</td><td>${sp.latex(dw)}$</td></tr>"
    table_html += f"<tr><td>$u_1$</td><td>${sp.latex(u1)}$</td></tr>"
    table_html += f"<tr><td>$u_2$</td><td>${sp.latex(u2)}$</td></tr>"
    table.innerHTML = table_html

    # Re-render LaTeX
    js.rerender()